from dataclasses import dataclass

import requests

from cy_weather_api.models import CyWeatherAPIResponseHandler, initFromDict

API_BASE = (
    "http://api.caiyunapp.com/v2.5/{token}/{lng},{lat}/weather.json?"
    "&lang={lang}"
    "&unit={unit}"
    "&dailysteps={dailysteps}"
    "&hourlysteps={hourlysteps}"
)
VALID_UNIT_OPTIONS = ["metric", "metric:v1", "metric:v2", "imperial", "SI"]
VALID_GRANU_OPTIONS = ["realtime", "minutely", "hourly", "daily", "weather"]


@dataclass
class CyWeatherAPIClient:
    token: str = "TAkhjf8d1nlSlspN"
    session = requests.Session()

    def fetch(
        self,
        lng: float,
        lat: float,
        lang: str = "en_US",
        begin: int = None,
        alert: bool = False,
        granu: str = None,
        unit: str = "metric",
        dailysteps: int = 5,
        hourlysteps: int = 48,
    ) -> CyWeatherAPIResponseHandler:

        if unit and unit not in VALID_UNIT_OPTIONS:
            raise ValueError(
                "Invaliad unit, got  {}, expect one from {}".format(
                    unit, VALID_UNIT_OPTIONS
                )
            )

        if granu and granu not in VALID_GRANU_OPTIONS:
            raise ValueError(
                "Invaliad granu, got  {}, expect one from {}".format(
                    unit, VALID_GRANU_OPTIONS
                )
            )

        if dailysteps < 1 or dailysteps > 15:
            raise ValueError("dailysteps in range [1, 15]")

        if hourlysteps < 1 or hourlysteps > 360:
            raise ValueError("hourlysteps in range [1, 360]")

        url = API_BASE.format(
            token=self.token,
            lng=lng,
            lat=lat,
            lang=lang,
            unit=unit,
            hourlysteps=hourlysteps,
            dailysteps=dailysteps,
        )

        if granu:
            url += "&granu={}".format(granu)

        if begin is not None:
            try:
                int(begin)
                url += "&begin={}".format(begin)
            except Exception:
                raise ValueError("Invalid begin")

        _data = self.session.get(url).json()
        if _data["status"] != "ok":
            _mess = (
                "Calling Caiyun API Errored with {error_mess}, please check token status\n"
                "url: {url}\n"
                "data: {api_data}\n"
            )
            mess = _mess.format(url=url, api_data=_data, error_mess=_data.get("error"))
            raise ValueError(mess)
        return initFromDict(_data)


if __name__ == "__main__":
    client = CyWeatherAPIClient(token="TAkhjf8d1nlSlspN")
    apiResult = client.fetch(lng=101.8551, lat=26.6832, lang="zh_CN", alert=True)
    print(apiResult.result.hourly.description)
    apiResult = client.fetch(lng=-0.2008, lat=51.5024, lang="en_GB")
    print(apiResult.result.hourly.description)
    apiResult = client.fetch(lng=73.9808, lat=40.7648, lang="en_US")
    print(apiResult.result.hourly.description)



from dataclasses import dataclass
from typing import List


@dataclass
class cyWeatherAPIResponseMinutelyStruct:
    status: str
    datasource: str
    precipitation_2h: List[float]
    precipitation: List[float]
    probability: List[float]
    description: str



from dataclasses import dataclass
from typing import List


@dataclass
class cyWeatherAPIResponseAlertContentItemStruct:
    province: str
    status: str
    code: str
    description: str
    pubtimestamp: float
    city: str
    adcode: str
    regionId: str
    latlon: List[float]
    county: str
    alertId: str
    request_status: str
    source: str
    title: str
    location: str

    def __post__init(self):
        self.lng = self.latlon[-1]
        self.lat = self.latlon[0]


@dataclass
class cyWeatherAPIResponseAlertStruct:
    status: str = None
    content: List[cyWeatherAPIResponseAlertContentItemStruct] = None



from dataclasses import dataclass
from typing import Any, List, Union


@dataclass
class cyWeatherAPIResponseHourlyWindStruct:
    datetime: str
    speed: Union[float, int]
    direction: Union[float, int]


@dataclass
class cyWeatherAPIResponseHourlyAirQualityItemValue:
    chn: float
    usa: float


@dataclass
class cyWeatherAPIResponseHourlyAirQualityItem:
    datetime: str
    value: cyWeatherAPIResponseHourlyAirQualityItemValue


@dataclass
class cyWeatherAPIResponseHourlyAirQualityPM25Item:
    datetime: str
    value: float


@dataclass
class cyWeatherAPIResponseHourlyAirQuality:
    aqi: List[cyWeatherAPIResponseHourlyAirQualityItem]
    pm25: List[cyWeatherAPIResponseHourlyAirQualityPM25Item]


@dataclass
class cyWeatherAPIResponseHourlySimpleDatetimeValuePair:
    datetime: str
    value: Any


@dataclass
class cyWeatherAPIResponseHourlyStruct:
    status: str
    description: str
    precipitation: List[cyWeatherAPIResponseHourlySimpleDatetimeValuePair]
    temperature: List[cyWeatherAPIResponseHourlySimpleDatetimeValuePair]
    humidity: List[cyWeatherAPIResponseHourlySimpleDatetimeValuePair]
    cloudrate: List[cyWeatherAPIResponseHourlySimpleDatetimeValuePair]
    skycon: List[cyWeatherAPIResponseHourlySimpleDatetimeValuePair]
    pressure: List[cyWeatherAPIResponseHourlySimpleDatetimeValuePair]
    visibility: List[cyWeatherAPIResponseHourlySimpleDatetimeValuePair]
    dswrf: List[cyWeatherAPIResponseHourlySimpleDatetimeValuePair]
    wind: List[cyWeatherAPIResponseHourlyWindStruct]
    air_quality: cyWeatherAPIResponseHourlyAirQuality



from dataclasses import dataclass

from cy_weather_api.models.alert import cyWeatherAPIResponseAlertStruct
from cy_weather_api.models.daily import cyWeatherAPIResponseDailyStruct
from cy_weather_api.models.hourly import cyWeatherAPIResponseHourlyStruct
from cy_weather_api.models.minutely import cyWeatherAPIResponseMinutelyStruct
from cy_weather_api.models.realtime import cyWeatherAPIResponseRealtimeStruct


@dataclass
class cyWeatherAPIResponseResultStruct:
    primary: int
    forecast_keypoint: str = None
    realtime: cyWeatherAPIResponseRealtimeStruct = None
    minutely: cyWeatherAPIResponseMinutelyStruct = None
    hourly: cyWeatherAPIResponseHourlyStruct = None
    daily: cyWeatherAPIResponseDailyStruct = None
    alert: cyWeatherAPIResponseAlertStruct = None



"""Dataclass for Caiyun Weather API.

Below dataclasses arr based on Caiyun Weather API v2.5, the offical API docs:

    zh_CN: https://open.caiyunapp.com/通用预报接口/v2.5
    en_US: https://open.caiyunapp.com/General_weather_interface/v2.5

We encourage use Golang's coding style:

    1. Use black as format tool.
    2. Use Camel-Case.
    3. Avoid complex oop tricks.
"""

import json
from dataclasses import asdict, dataclass, is_dataclass
from typing import Dict, List

import orjson
from dacite import from_dict

from cy_weather_api.models.result import cyWeatherAPIResponseResultStruct


class EnhancedJSONEncoder(json.JSONEncoder):
    def default(self, o):
        if is_dataclass(o):
            return asdict(o)
        return super().default(o)


@dataclass
class CyWeatherAPIResponseHandler:
    status: str
    api_version: str
    api_status: str
    lang: str
    unit: str
    tzshift: int
    timezone: str
    server_time: int
    location: List[float]
    result: cyWeatherAPIResponseResultStruct

    def __post_init__(self):
        pass

    def dumps(self, ensure_text=True) -> bytes:
        """Fast dumps via orjson.

        Check orjson docs for details: https://github.com/ijl/orjson#dataclass
        """
        if ensure_text:
            return orjson.dumps(self).decode("utf-8")
        else:
            return orjson.dumps(self)

    def dumps_pretty(self, **kwargs) -> str:
        """Dumps JSON with indent.

        Note: please do not change `cls` option if you don't know it.
        """
        return json.dumps(self, cls=EnhancedJSONEncoder, **kwargs)


def initFromDict(data: Dict) -> CyWeatherAPIResponseHandler:
    return from_dict(data_class=CyWeatherAPIResponseHandler, data=data)


if __name__ == "__main__":
    import requests

    # NOTE: Test token, no one can ensure its availability.
    TOKEN = "TAkhjf8d1nlSlspN"
    LNG, LAT = 116.3883, 39.9289
    URL = "http://api.caiyunapp.com/v2.5/{TOKEN}/{LNG},{LAT}/weather".format(
        TOKEN=TOKEN, LNG=LNG, LAT=LAT
    )

    apiResponseDataclass = from_dict(
        data_class=CyWeatherAPIResponseHandler, data=requests.get(URL).json()
    )
    print(apiResponseDataclass.dumps())



from dataclasses import dataclass
from typing import Union


@dataclass
class cyWeatherAPIResponseRealtimeWindStruct:
    speed: Union[float, int]
    direction: Union[float, int]


@dataclass
class cyWeatherAPIResponseRealtimePrecipitationItemLocalStruct:
    status: str
    datasource: str
    intensity: float


@dataclass
class cyWeatherAPIResponseRealtimePrecipitationItemNearestStruct:
    status: str
    distance: float
    intensity: float


@dataclass
class cyWeatherAPIResponseRealtimePrecipitationStruct:
    local: cyWeatherAPIResponseRealtimePrecipitationItemLocalStruct
    nearest: cyWeatherAPIResponseRealtimePrecipitationItemNearestStruct = None


@dataclass
class cyWeatherAPIResponseRealtimeAirQualityAQIValueStruct:
    chn: float
    usa: float


@dataclass
class cyWeatherAPIResponseRealtimeAirQualityAQIDescStruct:
    chn: str
    usa: str


@dataclass
class cyWeatherAPIResponseRealtimeAirQualityStruct:
    aqi: cyWeatherAPIResponseRealtimeAirQualityAQIValueStruct
    description: cyWeatherAPIResponseRealtimeAirQualityAQIDescStruct
    pm25: float = None
    pm10: float = None
    o3: float = None
    so2: float = None
    no2: float = None
    co: float = None


@dataclass
class cyWeatherAPIResponseRealtimeLifeIndexItemStruct:
    index: float
    desc: str


@dataclass
class cyWeatherAPIResponseRealtimeLifeIndexStruct:
    ultraviolet: cyWeatherAPIResponseRealtimeLifeIndexItemStruct
    comfort: cyWeatherAPIResponseRealtimeLifeIndexItemStruct


@dataclass
class cyWeatherAPIResponseRealtimeStruct:
    status: str
    temperature: float
    humidity: float
    cloudrate: float
    skycon: str
    visibility: float
    dswrf: float
    wind: cyWeatherAPIResponseRealtimeWindStruct
    pressure: float
    apparent_temperature: float
    precipitation: cyWeatherAPIResponseRealtimePrecipitationStruct
    air_quality: cyWeatherAPIResponseRealtimeAirQualityStruct
    life_index: cyWeatherAPIResponseRealtimeLifeIndexStruct



from dataclasses import dataclass
from typing import Any, List, Union


@dataclass
class cyWeatherAPIResponseDailyAstroItemTimeStruct:
    time: str


@dataclass
class cyWeatherAPIResponseDailyAstroItemStruct:
    date: str
    sunrise: cyWeatherAPIResponseDailyAstroItemTimeStruct
    sunset: cyWeatherAPIResponseDailyAstroItemTimeStruct


@dataclass
class cyWeatherAPIResponseDailyMaxMinAvgItemStruct:
    date: str
    max: float
    min: float
    avg: float


@dataclass
class cyWeatherAPIResponseDailyWindItemPropertyStruct:
    speed: Union[float, int]
    direction: Union[float, int]


@dataclass
class cyWeatherAPIResponseDailyWindItemStruct:
    date: str
    max: cyWeatherAPIResponseDailyWindItemPropertyStruct
    min: cyWeatherAPIResponseDailyWindItemPropertyStruct
    avg: cyWeatherAPIResponseDailyWindItemPropertyStruct


@dataclass
class cyWeatherAPIResponseDailyAirQualityItemAQITypeStruct:
    chn: float
    usa: float


@dataclass
class cyWeatherAPIResponseDailyAirQualityItemAQIStruct:
    date: str
    max: cyWeatherAPIResponseDailyAirQualityItemAQITypeStruct
    min: cyWeatherAPIResponseDailyAirQualityItemAQITypeStruct
    avg: cyWeatherAPIResponseDailyAirQualityItemAQITypeStruct


@dataclass
class cyWeatherAPIResponseDailyAirQualityItemStruct:
    aqi: List[cyWeatherAPIResponseDailyAirQualityItemAQIStruct]
    pm25: List[cyWeatherAPIResponseDailyMaxMinAvgItemStruct]


@dataclass
class cyWeatherAPIResponseDailySkyconItemStruct:
    date: str
    value: str


@dataclass
class cyWeatherAPIResponseDailyDateIndexDescItemStruct:
    date: str
    index: Any
    desc: str


@dataclass
class cyWeatherAPIResponseDailyLifeIndexItemStruct:
    ultraviolet: List[cyWeatherAPIResponseDailyDateIndexDescItemStruct]
    carWashing: List[cyWeatherAPIResponseDailyDateIndexDescItemStruct]
    dressing: List[cyWeatherAPIResponseDailyDateIndexDescItemStruct]
    comfort: List[cyWeatherAPIResponseDailyDateIndexDescItemStruct]
    coldRisk: List[cyWeatherAPIResponseDailyDateIndexDescItemStruct]


@dataclass
class cyWeatherAPIResponseDailyStruct:
    status: str
    astro: List[cyWeatherAPIResponseDailyAstroItemStruct]
    precipitation: List[cyWeatherAPIResponseDailyMaxMinAvgItemStruct]
    temperature: List[cyWeatherAPIResponseDailyMaxMinAvgItemStruct]
    humidity: List[cyWeatherAPIResponseDailyMaxMinAvgItemStruct]
    cloudrate: List[cyWeatherAPIResponseDailyMaxMinAvgItemStruct]
    pressure: List[cyWeatherAPIResponseDailyMaxMinAvgItemStruct]
    visibility: List[cyWeatherAPIResponseDailyMaxMinAvgItemStruct]
    dswrf: List[cyWeatherAPIResponseDailyMaxMinAvgItemStruct]
    wind: List[cyWeatherAPIResponseDailyWindItemStruct]
    skycon: List[cyWeatherAPIResponseDailySkyconItemStruct]
    skycon_08h_20h: List[cyWeatherAPIResponseDailySkyconItemStruct]
    skycon_20h_32h: List[cyWeatherAPIResponseDailySkyconItemStruct]
    life_index: cyWeatherAPIResponseDailyLifeIndexItemStruct



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cy_weather_api', 'cy_weather_api.models']

package_data = \
{'': ['*']}

install_requires = \
['dacite>=1.5.0,<2.0.0', 'orjson>=3.0.0,<4.0.0', 'requests>=2.23.0,<3.0.0']

setup_kwargs = {
    'name': 'cy-weather-api',
    'version': '0.1.5',
    'description': 'Caiyun Weather API Python SDK',
    'long_description': '# Caiyun Weather API Python SDK\n\n## Install\n\nPython 3.6+ is required.\n\n```sh\npip install cy-weather-api\n```\n\n## Usage\n\n### Request Caiyun API\n\n```py\nfrom cy_weather_api import CyWeatherAPIClient\n\nclient = CyWeatherAPIClient(token="TAkhjf8d1nlSlspN")\napiResult = client.fetch(lng=101.8551, lat=26.6832, lang="zh_CN", alert=True)\nprint(apiResult.result.hourly.description)\napiResult = client.fetch(lng=-0.2008, lat=51.5024, lang="en_GB")\nprint(apiResult.result.hourly.description)\napiResult = client.fetch(lng=73.9808, lat=40.7648, lang="en_US")\nprint(apiResult.result.hourly.description)\n```\n\nOutput sample:\n\n```\n\xe6\x99\xb4\xef\xbc\x8c\xe4\xbb\x8a\xe5\xa4\xa9\xe6\x99\x9a\xe9\x97\xb420\xe7\x82\xb9\xe9\x92\x9f\xe5\x90\x8e\xe8\xbd\xac\xe5\xb0\x8f\xe9\x9b\xa8\xef\xbc\x8c\xe5\x85\xb6\xe5\x90\x8e\xe5\xa4\x9a\xe4\xba\x91\nclear weather over the next 24 hours\nclear weather, overcast after 20 o\'clock this afternoon, followed by cloudy\n```\n\n### Use our dataclass models\n\nThe default HTTP client is requests, you can other HTTP cient to request API,\nand pass the response dict to our models (based on `dataclasss`):\n\n```py\nfrom cy_weather_api import initFromDict\n\ndata = {\n    "status": "ok",\n    "api_version": "v2.5",\n    "api_status": "active",\n    "lang": "en_US",\n    "unit": "metric",\n    "tzshift": 28800,\n    "timezone": "Asia/Shanghai",\n    "server_time": 1589125757,\n    "location": [39.888888, 116.674501],\n    "result": {"forecast_keypoint": "test forecast_keypoint", "primary": 0},\n}\napiResult = initFromDict(data)\n```\n',
    'author': 'Caiyunapp',
    'author_email': 'admin@caiyunapp.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/caiyunapp/caiyun-weather-api-python-sdk',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)
